@@ -3,7 +3,7 @@
 __author__ = 'DLTK'
 __email__ = 'connect@qubitai.tech'
 __copyright__ = 'Copyright (c) 2019-2020 The QubitAI Technologies LLC'
-__version__ = '1.0.1'
+__version__ = '1.0.3'
 __url__ = 'https://github.com/dltk-ai/qubitai-dltk'
 __download_url__ = ''
 __description__ = 'A Python wrapper around the DLTK API'
@@ -14,6 +14,10 @@ import json
 import requests
 import warnings
 
+def custom_formatwarning(msg, *args, **kwargs):
+    # ignore everything except the message
+    return str(msg) + '\n'
+
 try:
     url = f"https://pypi.org/pypi/qubitai-dltk/json"
     data = json.loads(requests.get(url).text)
@@ -21,6 +25,7 @@ try:
     installed_version = __version__
     if str(installed_version) != str(latest_version):
         warnings.filterwarnings('ignore', '.*do not.*')
-        warnings.warn(f'New version of dltk_ai ({latest_version}) available, you are still using older ({installed_version}) version of the dltk_ai', FutureWarning)
+        warnings.formatwarning = custom_formatwarning
+        warnings.warn(f'New version of dltk_ai ({latest_version}) available, you are still using older ({installed_version}) version of the dltk_ai, Please update using "pip install qubitai-dltk=={latest_version}"', FutureWarning)
 except:
     pass
